#define OLC_PGE_APPLICATION
#include "olcPixelGameEngine.h"
#include "Polymorphism.h"
#include "Player.h"
#include "Enemy.h"


Polymorphism::Polymorphism()
{
	// Name your application
	sAppName = "Polymorphism Example";
}

bool Polymorphism::OnUserCreate()
{
	player_img.Load("player.png");
	objects.push_back(new Player({16,16},0,{0,0},player_img));
	objects.push_back(new Enemy({64,64},0,{0,0},player_img));
	objects.push_back(new Enemy({32,108},0,{0,0},player_img));
	objects.push_back(new Enemy({48,86},0,{0,0},player_img));
	objects.push_back(new Enemy({128,156},0,{0,0},player_img));
	return true;
}

bool Polymorphism::OnUserUpdate(float fElapsedTime){
	for(Object*obj:objects){
		obj->Update(this,fElapsedTime);
		obj->InternalUpdate(this,fElapsedTime);
	}

	for(int listMarker=0;listMarker<objects.size();listMarker++){
		for(int objMarker=listMarker;objMarker<objects.size();objMarker++){
			if(listMarker==objMarker)continue;
			auto dist = [&](olc::vf2d pos1,olc::vf2d pos2){return sqrt(pow(pos1.x-pos2.x,2)+pow(pos1.y-pos2.y,2));};
			Object*obj1=objects[listMarker];
			Object*obj2=objects[objMarker];
			if(dist(obj1->pos,obj2->pos)<(obj1->radius+obj2->radius)){
				obj1->Collision(obj2);
				obj2->Collision(obj1);
			}
		}
	}

	for(Object*obj:objects){
		obj->Draw(this,fElapsedTime);
	}

	std::erase_if(objects,[&](Object*obj){
		if(obj->dead){
			delete obj;
			return true;
		} else {
			return false;
		}
	});
	return true;
}

int main()
{
	Polymorphism demo;
	if (demo.Construct(256, 240, 4, 4))
		demo.Start();
	return 0;
}